package com.ematou.wxbase.service;

import com.ematou.wxbase.common.utils.DateUtils;
import com.ematou.wxbase.config.WxGeneralConfig;
import com.ematou.wxbase.entity.OperationRecord;
import com.ematou.wxbase.entity.TokenRecord;
import com.ematou.wxbase.mapper.TokenRecordMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.client.RestTemplate;

import java.text.ParseException;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author lhm
 * @version 1.0
 * 2021-04-30 17:53
 */
@Service
public class TokenRecordService {

    private static final Logger logger = LoggerFactory.getLogger(TokenRecordService.class);

    @Autowired
    OperationRecordService operationRecordService;

    @Autowired
    TokenRecordMapper tokenRecordMapper;

    @Autowired
    RestTemplate restTemplate;

    @Autowired
    WxGeneralConfig wxGeneralConfig;

    /**
     * 查询所有未失效的Token记录
     * @return 所有未失效的Token记录
     */
    public List<TokenRecord> queryTokenIsValid() {
        try {
            return tokenRecordMapper.queryTokenIsValid();
        } catch (RuntimeException e) {
            logger.error("查询所有未失效的Token记录失败！errorMessage：" + e.getMessage());
            return null;
        }
    }

    /**
     * 查询最新且有效的Token记录总数
     * @return token记录总数
     */
    public int queryLatestAndValidRecordCount() {
        try {
            return tokenRecordMapper.queryLatestAndValidRecordCount();
        } catch (RuntimeException e) {
            logger.error("查询最新且有效的Token记录总数失败！errorMessage：" + e.getMessage());
            return -1;
        }
    }

    /**
     * 新增一条Token记录
     * @param tokenRecord token记录
     */
    public void insertRecord(TokenRecord tokenRecord) {
        if (null == tokenRecord) {
            logger.error("Token记录为空，插入失败！");
            return;
        }
        try {
            tokenRecordMapper.insertRecord(tokenRecord);
        } catch (RuntimeException e) {
            logger.error("插入Token记录失败！errorMessage：" + e.getMessage());
        }
    }

    /**
     * 获取数据库中最新且未过期的AccessToken
     *
     * @return token
     */
    public TokenRecord getLatestToken() throws ParseException {
        // 如果数据库中没有或者是该token已过期，则需要重新生成token
        TokenRecord tokenRecord = tokenRecordMapper.queryLatestAndValidRecord();
        if (null == tokenRecord) {
            // 需要生成Token
            return generateToken();
        }
        String expiresTime = tokenRecord.getExpiresTime();
        Date tokenExpiresTime = DateUtils.parseString(expiresTime);
        Date now = new Date();
        if (now.getTime() > tokenExpiresTime.getTime()) {
            return generateToken();
        }
        return tokenRecord;
    }

    /**
     * 主动请求微信生成accessToken
     *
     * @return token记录
     */
    public TokenRecord generateToken() throws RuntimeException {

        String url = String.format(wxGeneralConfig.getUrl(), wxGeneralConfig.getGrantType(), wxGeneralConfig.getAppId(), wxGeneralConfig.getAppSecret());

        Map response = null;
        try {
            response = restTemplate.getForEntity(url, Map.class).getBody();
        } catch (Exception e) {
            logger.error("请求微信平台失败！errorMessage：" + e.getMessage());
            throw new RuntimeException("请求微信平台失败！");
        }
        if (null == response) {
            logger.error("请求微信平台失败！errorMessage：response is null");
            throw new RuntimeException("请求微信平台失败！");
        }

        OperationRecord operationRecord = wrapOperationRecord("目前暂无");
        TokenRecord tokenRecord = wrapTokenRecord(operationRecord, response);

        if (null == tokenRecord) {
            Object errcode = response.get("errcode");
            Object errmsg = response.get("errmsg");
            logger.error("请求微信平台生成AccessToken失败！errorCode：" + errcode + "，errorMessage：" + errmsg);
            throw new RuntimeException("请求微信平台生成AccessToken失败！");
        }

        // 保存操作记录
        operationRecordService.insertRecord(operationRecord);
        int latestId = operationRecordService.queryLatestId();
        // 保存TokenRecord
        tokenRecord.setOpId(latestId);
        insertRecord(tokenRecord);
        return tokenRecord;
    }

    /**
     * 批量更新Token的状态
     * @param resultList id数组
     */
    public void updateBatchRecord(List<Integer> resultList) {
        if (CollectionUtils.isEmpty(resultList)) {
            logger.warn("需要批量更新的Token记录集合为空，无需更新！");
            return;
        }
        try {
            tokenRecordMapper.updateBatchRecord(resultList);
        } catch (RuntimeException e) {
            logger.error("批量更新Token记录状态失败！errorMessage：" + e.getMessage());
            throw new RuntimeException("批量更新Token记录状态失败！");
        }
    }

    /**
     * 包装操作记录
     *
     * @param appCode 应用编码
     * @return 操作记录
     */
    private OperationRecord wrapOperationRecord(String appCode) {
        OperationRecord operationRecord = new OperationRecord();
        operationRecord.setAppCode(appCode);
        return operationRecord;
    }

    /**
     * 包装Token记录
     *
     * @param operationRecord 操作记录
     * @param response        请求微信平台响应
     * @return Token记录
     */
    private TokenRecord wrapTokenRecord(OperationRecord operationRecord, Map response) {
        TokenRecord tokenRecord = new TokenRecord();
        String accessToken = (String) response.get("access_token");
        Integer expiresIn = (Integer) response.get("expires_in");
        Integer errCode = (Integer) response.get("errcode");
        String errMsg = (String) response.get("errmsg");

        if (StringUtils.hasLength(accessToken) && expiresIn != null && errCode == null && !StringUtils.hasLength(errMsg)) {
            Date now = new Date();
            long expiresTime = now.getTime() + (expiresIn * 1000);
            tokenRecord.setAccessToken(accessToken);
            tokenRecord.setExpiresIn(expiresIn);
            tokenRecord.setEffectTime(DateUtils.formatDate(now));
            tokenRecord.setExpiresTime(DateUtils.formatDate(new Date(expiresTime)));
            tokenRecord.setOpId(operationRecord.getOpId());
            tokenRecord.setIsValid(0);
            return tokenRecord;
        }
        return null;
    }

}
